@@ -1,10 +1,10 @@
 /*
- * Nautilus Actions
+ * Nautilus-Actions
  * A Nautilus extension which offers configurable context menu actions.
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,6 +34,8 @@
 
 #include <api/na-iduplicable.h>
 
+#include "na-marshal.h"
+
 /* private interface data
  */
 struct _NAIDuplicableInterfacePrivate {
@@ -46,7 +48,6 @@ typedef struct {
 	NAIDuplicable *origin;
 	gboolean       modified;
 	gboolean       valid;
-	gulong         status_changed_handler_id;
 }
 	DuplicableStr;
 
@@ -55,7 +56,8 @@ typedef struct {
 /* signals emitted on NAIDuplicable when a status changes
  */
 enum {
-	STATUS_CHANGED,
+	MODIFIED_CHANGED,
+	VALID_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -74,8 +76,9 @@ static gboolean       v_is_valid( const NAIDuplicable *object );
 
 static DuplicableStr *get_duplicable_str( const NAIDuplicable *object );
 
-static void           status_changed_handler( NAIDuplicable *instance, gpointer user_data );
-static void           propagate_signal_to_consumers( const gchar *signal, NAIDuplicable *instance, gpointer user_data );
+static void           on_modified_changed_class_handler( NAIDuplicable *instance, GObject *object, gboolean is_modified );
+static void           on_valid_changed_class_handler( NAIDuplicable *instance, GObject *object, gboolean is_valid );
+static void           propagate_signal_to_consumers( NAIDuplicable *instance, const gchar *signal, GObject *object, gboolean new_status );
 static void           release_signal_consumers( GList *consumers );
 
 GType
@@ -135,22 +138,68 @@ interface_base_init( NAIDuplicableInterface *klass )
 		klass->is_valid = NULL;
 
 		/**
-		 * NAIDuplicable::status-changed:
+		 * NAIDuplicable::modified-changed:
 		 *
 		 * This signal is emitted by #NAIDuplicable when the modification
-		 * or the validity status of an object has been modified.
+		 * status of an object has been modified.
+		 *
+		 * The default class handler propagates the signal to registered
+		 * consumers.
+		 *
+		 * Signal args: New modification status
+		 *
+		 * Handler prototype:
+		 * void ( *handler )( NAIDuplicable *duplicable, NAObject *object, gboolean is_modified, gpointer user_data );
+		 *
+		 * When the signal is first emitted, thus on NAIDuplicable, @duplicable
+		 * and @object are pointers to the same address. This duplication is
+		 * relevant when propagating the signal to customer, as the signal is
+		 * emitted on the customer itself, while we still need the @object
+		 * pointer.
 		 */
-		st_signals[ STATUS_CHANGED ] = g_signal_new(
-				NA_IDUPLICABLE_SIGNAL_STATUS_CHANGED,
+		st_signals[ MODIFIED_CHANGED ] = g_signal_new_class_handler(
+				IDUPLICABLE_SIGNAL_MODIFIED_CHANGED,
 				G_TYPE_OBJECT,
-				G_SIGNAL_RUN_LAST,
-				0,						/* no default handler */
+				G_SIGNAL_RUN_CLEANUP,
+				G_CALLBACK( on_modified_changed_class_handler ),
 				NULL,
 				NULL,
-				g_cclosure_marshal_VOID__POINTER,
+				na_cclosure_marshal_VOID__POINTER_BOOLEAN,
 				G_TYPE_NONE,
-				1,
-				G_TYPE_POINTER );
+				2,
+				G_TYPE_POINTER, G_TYPE_BOOLEAN );
+
+		/**
+		 * NAIDuplicable::valid-changed:
+		 *
+		 * This signal is emitted by #NAIDuplicable when the validity
+		 * status of an object has been modified.
+		 *
+		 * The default class handler propagates the signal to registered
+		 * consumers.
+		 *
+		 * Signal args: New validity status
+		 *
+		 * Handler prototype:
+		 * void ( *handler )( NAIDuplicable *duplicable, NAObject *object, gboolean is_valid, gpointer user_data );
+		 *
+		 * When the signal is first emitted, thus on NAIDuplicable, @duplicable
+		 * and @object are pointers to the same address. This duplication is
+		 * relevant when propagating the signal to customer, as the signal is
+		 * emitted on the customer itself, while we still need the @object
+		 * pointer.
+		 */
+		st_signals[ VALID_CHANGED ] = g_signal_new_class_handler(
+				IDUPLICABLE_SIGNAL_VALID_CHANGED,
+				G_TYPE_OBJECT,
+				G_SIGNAL_RUN_CLEANUP,
+				G_CALLBACK( on_valid_changed_class_handler ),
+				NULL,
+				NULL,
+				na_cclosure_marshal_VOID__POINTER_BOOLEAN,
+				G_TYPE_NONE,
+				2,
+				G_TYPE_POINTER, G_TYPE_BOOLEAN );
 
 		st_interface = klass;
 
@@ -193,12 +242,8 @@ na_iduplicable_dispose( const NAIDuplicable *object )
 	if( st_initialized && !st_finalized ){
 
 		str = get_duplicable_str( object );
-
-		if( g_signal_handler_is_connected(( gpointer ) object, str->status_changed_handler_id )){
-			g_signal_handler_disconnect(( gpointer ) object, str->status_changed_handler_id );
-		}
-
 		g_free( str );
+		g_object_set_data( G_OBJECT( object ), NA_IDUPLICABLE_DATA_DUPLICABLE, NULL );
 	}
 }
 
@@ -228,9 +273,9 @@ na_iduplicable_dump( const NAIDuplicable *object )
 
 		str = get_duplicable_str( object );
 
-		g_debug( "%s:   origin=%p", thisfn, ( void * ) str->origin );
-		g_debug( "%s: modified=%s", thisfn, str->modified ? "True" : "False" );
-		g_debug( "%s:    valid=%s", thisfn, str->valid ? "True" : "False" );
+		g_debug( "| %s:   origin=%p", thisfn, ( void * ) str->origin );
+		g_debug( "| %s: modified=%s", thisfn, str->modified ? "True" : "False" );
+		g_debug( "| %s:    valid=%s", thisfn, str->valid ? "True" : "False" );
 	}
 }
 
@@ -304,8 +349,8 @@ na_iduplicable_duplicate( const NAIDuplicable *object )
  * na_iduplicable_check_status() is not, as itself, recursive.
  * That is, the modification and validity status are only set on the
  * specified object.
- * #NAObject implementation has choosen to handle itself the recursivity:
- * na_object_check_status() so first check status for childs, before
+ * #NAObject implementation has chosen to handle itself the recursivity:
+ * na_object_check_status() so first check status for children, before
  * calling this function.
  *
  * Since: 2.30
@@ -316,22 +361,19 @@ na_iduplicable_check_status( const NAIDuplicable *object )
 	static const gchar *thisfn = "na_iduplicable_check_status";
 	DuplicableStr *str;
 	gboolean was_modified, was_valid;
-	gboolean changed;
 
 	g_return_if_fail( NA_IS_IDUPLICABLE( object ));
 
 	if( st_initialized && !st_finalized ){
-
 		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
 
 		str = get_duplicable_str( object );
 
 		was_modified = str->modified;
 		was_valid = str->valid;
-		changed = FALSE;
 
 		if( str->origin ){
-			g_debug( "%s: origin=%p (%s)", thisfn, ( void * ) str->origin, G_OBJECT_TYPE_NAME( str->origin ));
+			g_debug( "%s: vs. origin=%p (%s)", thisfn, ( void * ) str->origin, G_OBJECT_TYPE_NAME( str->origin ));
 			g_return_if_fail( NA_IS_IDUPLICABLE( str->origin ));
 			str->modified = !v_are_equal( str->origin, object );
 
@@ -339,37 +381,19 @@ na_iduplicable_check_status( const NAIDuplicable *object )
 			str->modified = TRUE;
 		}
 
+		if( was_modified != str->modified ){
+			g_debug( "%s: %p (%s) status changed to modified=%s",
+					thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ), str->modified ? "True":"False" );
+			g_signal_emit_by_name( G_OBJECT( object ), IDUPLICABLE_SIGNAL_MODIFIED_CHANGED, object, str->modified );
+		}
+
 		str->valid = v_is_valid( object );
 
-		if( was_modified && !str->modified ){
-			g_debug( "%s: %p (%s) status changed to non-modified",
-					thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-			changed = TRUE;
-		} else if ( !was_modified && str->modified ){
-			g_debug( "%s: %p (%s) status changed to modified",
-					thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-			changed = TRUE;
+		if( was_valid != str->valid ){
+			g_debug( "%s: %p (%s) status changed to valid=%s",
+					thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ), str->valid ? "True":"False" );
+			g_signal_emit_by_name( G_OBJECT( object ), IDUPLICABLE_SIGNAL_VALID_CHANGED, object, str->valid );
 		}
-
-		if( was_valid && !str->valid ){
-			g_debug( "%s: %p (%s) status changed to non-valid",
-					thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-			changed = TRUE;
-		} else if( !was_valid && str->valid ){
-			g_debug( "%s: %p (%s) status changed to valid",
-					thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-			changed = TRUE;
-		}
-
-		if( changed ){
-			g_signal_emit_by_name( G_OBJECT( object ), NA_IDUPLICABLE_SIGNAL_STATUS_CHANGED, object );
-		}
-
-#if 0
-		g_debug( "%s: object=%p (%s), modified=%s, valid=%s", thisfn,
-				( void * ) object, G_OBJECT_TYPE_NAME( object ),
-				modified ? "True":"False", valid ? "True":"False" );
-#endif
 	}
 }
 
@@ -487,6 +511,7 @@ na_iduplicable_set_origin( NAIDuplicable *object, const NAIDuplicable *origin )
 	}
 }
 
+#ifndef NA_DISABLE_DEPRECATED
 /**
  * na_iduplicable_set_modified:
  * @object: the #NAIDuplicable object whose modification status is to be set.
@@ -495,6 +520,7 @@ na_iduplicable_set_origin( NAIDuplicable *object, const NAIDuplicable *origin )
  * Sets the new modified of a duplicated #NAIDuplicable.
  *
  * Since: 2.30
+ * Deprecated: 3.1.0
  */
 void
 na_iduplicable_set_modified( NAIDuplicable *object, gboolean modified )
@@ -506,13 +532,10 @@ na_iduplicable_set_modified( NAIDuplicable *object, gboolean modified )
 	if( st_initialized && !st_finalized ){
 
 		str = get_duplicable_str( object );
-
-		if( str->modified != modified ){
-			str->modified = modified;
-			g_signal_emit_by_name( G_OBJECT( object ), NA_IDUPLICABLE_SIGNAL_STATUS_CHANGED, object );
-		}
+		str->modified = modified;
 	}
 }
+#endif /* NA_DISABLE_DEPRECATED */
 
 static void
 v_copy( NAIDuplicable *target, const NAIDuplicable *source )
@@ -529,7 +552,7 @@ v_are_equal( const NAIDuplicable *a, const NAIDuplicable *b )
 		return( NA_IDUPLICABLE_GET_INTERFACE( a )->are_equal( a, b ));
 	}
 
-	return( FALSE );
+	return( TRUE );
 }
 
 static gboolean
@@ -539,7 +562,7 @@ v_is_valid( const NAIDuplicable *object )
 		return( NA_IDUPLICABLE_GET_INTERFACE( object )->is_valid( object ));
 	}
 
-	return( FALSE );
+	return( TRUE );
 }
 
 /**
@@ -565,20 +588,23 @@ na_iduplicable_register_consumer( GObject *consumer )
 }
 
 static void
-status_changed_handler( NAIDuplicable *instance, gpointer user_data )
+on_modified_changed_class_handler( NAIDuplicable *instance, GObject *object, gboolean is_modified )
 {
-	/*g_debug( "na_iduplicable_propagate_modified_changed: instance=%p (%s), user_data=%p (%s)",
-			( void * ) instance, G_OBJECT_TYPE_NAME( instance ),
-			( void * ) user_data, G_OBJECT_TYPE_NAME( user_data ));*/
-
-	propagate_signal_to_consumers( NA_IDUPLICABLE_SIGNAL_STATUS_CHANGED, instance, user_data );
+	if( NA_IS_IDUPLICABLE( instance )){
+		propagate_signal_to_consumers( instance, IDUPLICABLE_SIGNAL_MODIFIED_CHANGED, object, is_modified );
+	}
 }
 
-/*
- * note that propagating the signal to consumers re-triggers
- */
 static void
-propagate_signal_to_consumers( const gchar *signal, NAIDuplicable *instance, gpointer user_data )
+on_valid_changed_class_handler( NAIDuplicable *instance, GObject *object, gboolean is_valid )
+{
+	if( NA_IS_IDUPLICABLE( instance )){
+		propagate_signal_to_consumers( instance, IDUPLICABLE_SIGNAL_VALID_CHANGED, object, is_valid );
+	}
+}
+
+static void
+propagate_signal_to_consumers( NAIDuplicable *instance, const gchar *signal, GObject *object, gboolean new_status )
 {
 	static const gchar *thisfn = "na_iduplicable_propagate_signals_to_consumers";
 	GList *ic;
@@ -586,11 +612,10 @@ propagate_signal_to_consumers( const gchar *signal, NAIDuplicable *instance, gpo
 	g_return_if_fail( st_interface );
 
 	if( st_initialized && !st_finalized ){
-
-		g_debug( "%s: signal=%s, instance=%p, user_data=%p", thisfn, signal, ( void * ) instance, ( void * ) user_data );
+		g_debug( "%s: instance=%p, signal=%s", thisfn, ( void * ) instance, signal );
 
 		for( ic = st_interface->private->consumers ; ic ; ic = ic->next ){
-			g_signal_emit_by_name( ic->data, signal, user_data );
+			g_signal_emit_by_name( ic->data, signal, object, new_status );
 		}
 	}
 }
@@ -615,12 +640,6 @@ get_duplicable_str( const NAIDuplicable *object )
 		str->modified = FALSE;
 		str->valid = TRUE;
 
-		str->status_changed_handler_id = g_signal_connect(
-				G_OBJECT( object ),
-				NA_IDUPLICABLE_SIGNAL_STATUS_CHANGED,
-				G_CALLBACK( status_changed_handler ),
-				( gpointer ) object );
-
 		g_object_set_data( G_OBJECT( object ), NA_IDUPLICABLE_DATA_DUPLICABLE, str );
 	}
 
