@@ -57,8 +57,6 @@ static void     instance_init( GTypeInstance *instance, gpointer klass );
 static void     instance_dispose( GObject *object );
 static void     instance_finalize( GObject *object );
 
-static gboolean object_is_valid( const NAObject *object );
-
 static gchar   *v_new_id( const NAObjectId *object, const NAObjectId *new_parent );
 
 GType
@@ -103,7 +101,6 @@ class_init( NAObjectIdClass *klass )
 {
 	static const gchar *thisfn = "na_object_id_class_init";
 	GObjectClass *object_class;
-	NAObjectClass *naobject_class;
 
 	g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
@@ -113,28 +110,18 @@ class_init( NAObjectIdClass *klass )
 	object_class->dispose = instance_dispose;
 	object_class->finalize = instance_finalize;
 
-	naobject_class = NA_OBJECT_CLASS( klass );
-	naobject_class->dump = NULL;
-	naobject_class->copy = NULL;
-	naobject_class->are_equal = NULL;
-	naobject_class->is_valid = object_is_valid;
-
 	klass->private = g_new0( NAObjectIdClassPrivate, 1 );
 }
 
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
-	static const gchar *thisfn = "na_object_id_instance_init";
 	NAObjectId *self;
 
 	g_return_if_fail( NA_IS_OBJECT_ID( instance ));
 
 	self = NA_OBJECT_ID( instance );
 
-	g_debug( "%s: instance=%p (%s), klass=%p",
-			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
-
 	self->private = g_new0( NAObjectIdPrivate, 1 );
 }
 
@@ -157,8 +144,6 @@ instance_dispose( GObject *object )
 
 	if( !self->private->dispose_has_run ){
 
-		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 		self->private->dispose_has_run = TRUE;
 
 		parent = na_object_get_parent( object );
@@ -195,26 +180,6 @@ instance_finalize( GObject *object )
 	}
 }
 
-/*
- * a NAObjectId is valid if it has a non-null id
- */
-static gboolean
-object_is_valid( const NAObject *object )
-{
-	gboolean is_valid;
-	gchar *id;
-
-	is_valid = TRUE;
-
-	if( is_valid ){
-		id = na_object_get_id( object );
-		is_valid = ( id != NULL && strlen( id ) > 0 );
-		g_free( id );
-	}
-
-	return( is_valid );
-}
-
 /**
  * na_object_id_sort_alpha_asc:
  * @a: first #NAObjectId.
@@ -377,7 +342,7 @@ na_object_id_set_copy_of_label( NAObjectId *object )
 
 /**
  * na_object_id_set_new_id:
- * @object: the #NAObjectId object whose internal identifiant is to be
+ * @object: the #NAObjectId object whose internal identifier is to be
  * set.
  * @new_parent: if @object is a #NAObjectProfile, then @new_parent
  * should be set to the #NAObjectAction new parent. Else, it would not
@@ -415,27 +380,11 @@ na_object_id_set_new_id( NAObjectId *object, const NAObjectId *new_parent )
 static gchar *
 v_new_id( const NAObjectId *object, const NAObjectId *new_parent )
 {
-	gchar *new_id;
-	GList *hierarchy, *ih;
-	gboolean found;
+	gchar *new_id = NULL;
 
-	found = FALSE;
-	new_id = NULL;
-	hierarchy = g_list_reverse( na_object_get_hierarchy( NA_OBJECT( object )));
-	/*g_debug( "na_object_id_most_derived_id: object=%p (%s)",
-					( void * ) object, G_OBJECT_TYPE_NAME( object ));*/
-
-	for( ih = hierarchy ; ih && !found ; ih = ih->next ){
-		if( NA_OBJECT_ID_CLASS( ih->data )->new_id ){
-			new_id = NA_OBJECT_ID_CLASS( ih->data )->new_id( object, new_parent );
-			found = TRUE;
-		}
-		if( G_OBJECT_CLASS_TYPE( ih->data ) == NA_OBJECT_ID_TYPE ){
-			break;
-		}
+	if( NA_OBJECT_ID_GET_CLASS( object )->new_id ){
+		new_id = NA_OBJECT_ID_GET_CLASS( object )->new_id( object, new_parent );
 	}
 
-	na_object_free_hierarchy( hierarchy );
-
 	return( new_id );
 }
